--[[ ---------------------------------------------------------------------------

BuffEnough: personal buff monitor.

BuffEnough is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

BuffEnough is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BuffEnough.	If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]


BuffEnough = nil

if LibStub("LibLogger-1.0", true) then
    BuffEnough = LibStub("AceAddon-3.0"):NewAddon("BuffEnough", "AceEvent-3.0", "AceBucket-3.0", "AceConsole-3.0", "LibLogger-1.0")
else
	BuffEnough = LibStub("AceAddon-3.0"):NewAddon("BuffEnough", "AceEvent-3.0", "AceBucket-3.0", "AceConsole-3.0")
end

local L = LibStub("AceLocale-3.0"):GetLocale("BuffEnough")
local TQ = LibStub:GetLibrary("LibTalentQuery-1.0")
local G = LibStub:GetLibrary("LibGratuity-3.0")


local invSlots = {"HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "WristSlot", "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "Trinket0Slot", "Trinket1Slot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot"}

local warnings = {}
warnings[L["Missing"]] = false
warnings[L["Broken"]] = false
warnings[L["Unexpected"]] = false
warnings[L["Low"]] = true
warnings[L["Unhappy"]] = false


--[[ ---------------------------------------------------------------------------
	 Ace3 initialization
----------------------------------------------------------------------------- ]]
function BuffEnough:OnInitialize()

	-- Initialize persistent addon variables
	self.raidClassCount = {}
	self.partyClassCount = {}
	self.talents = {}
	self.talentsAvailable = {}
	self.durations = {}
	self.lastBuffer = {}
	self.playerIsTank = false
		
	self.trackedItems = {}
	self.results = {}

	self.Anchor = nil
	self.Display = nil
	self.Grip = nil
	self.tooltip = ""
	self.optionsFrame = nil
	self.isShowingTooltip = false
	self.timeSinceBuffEnough = 0

	self.isBuffEnough = true
	self.isBuffWarning = false

	-- Set up saved variables and config options
	self.db = LibStub("AceDB-3.0"):New("BuffEnoughDB", self.defaults, "Default")

	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileDeleted","OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable(L["BuffEnough"], self.options, {L["buffenough"], L["be"]})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(L["BuffEnough"], L["BuffEnough"])

	TQ.RegisterCallback(self, "TalentQuery_Ready")

	-- Logging
	if LibStub("LibLogger-1.0", true) then
		self:SetLogLevel(self.db.profile.logLevel)
		self:SetPerformanceMode(true)
	end

	-- Set up display
	self:CreateFrame()

end


--[[ ---------------------------------------------------------------------------
	 Ace3 enable addon
----------------------------------------------------------------------------- ]]
function BuffEnough:OnEnable()

	self:DoEnable()

end


--[[ ---------------------------------------------------------------------------
	 Enables addon if option 'show' is true
----------------------------------------------------------------------------- ]]
function BuffEnough:DoEnable()

	if self:GetProfileParam("show") then

		if self.debug then self:debug("Addon enabled") end

		self:RegisterBucketEvent("PLAYER_AURAS_CHANGED", .2, "RunCheck")
		self:RegisterBucketEvent("UNIT_INVENTORY_CHANGED", .2, "RunCheck")
		self:RegisterBucketEvent("UPDATE_INVENTORY_ALERTS", .2, "RunCheck")
		self:RegisterBucketEvent("RAID_ROSTER_UPDATE", .2, "RunCheck")
		self:RegisterBucketEvent("PARTY_MEMBERS_CHANGED", .2, "RunCheck")

		self:RegisterEvent("UNIT_PET", "RunCheck")
		self:RegisterEvent("ZONE_CHANGED", "RunCheck")
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "RunCheck")
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "RunCheck")
		self:RegisterEvent("READY_CHECK", "RunCheck")
		
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "RecordLastBuffer")

		if oRA then
			self:RegisterEvent("oRA_MainTankUpdate", "RunCheck")
			if self.debug then self:debug("Registered for oRA MT updates") end
		elseif CT_RA_MainTanks then
			hooksecurefunc("CT_RAOptions_UpdateMTs", function() self:RunCheck() end)
			if self.debug then self:debug("Registered for CTRaid MT updates") end
		end

		self:SetAnchors(true)
		self:UpdateVisible()

		self:RunCheck()

	end

end


--[[ ---------------------------------------------------------------------------
	 Ace3 disable addon
----------------------------------------------------------------------------- ]]
function BuffEnough:OnDisable()

	self:DoDisable()

end


--[[ ---------------------------------------------------------------------------
	 Disables addon
----------------------------------------------------------------------------- ]]
function BuffEnough:DoDisable()

	if self.debug then self:debug("Addon disabled") end

	self:UnregisterAllBuckets()
	self:UnregisterAllEvents()
	self:UpdateVisible()

end


--[[ ---------------------------------------------------------------------------
	 Toggling the display basically toggles the addon, so if the display is
	 shown, we enable the addon, if the display is hidden, we disable the
	 addon
----------------------------------------------------------------------------- ]]
function BuffEnough:ToggleDisplay()

	self:SetProfileParam("show", not self:GetProfileParam("show"))

	if self:GetProfileParam("show") then
		self:DoEnable()
	else
		self:DoDisable()
	end

end


--[[ ---------------------------------------------------------------------------
	 Run the check!
----------------------------------------------------------------------------- ]]
function BuffEnough:RunCheck()

	if self.debug then self:debug("Running check") end

	self.tooltip = ""
	self.trackedItems = {}
	self.results = {}
	self.isBuffEnough = true
	self.isBuffWarning = false

	self:CheckTank()
	self:ScanRaidParty()
	self:CheckBuffs()
	self:GetModule("Player"):CheckClassBuffs()
	self:GetModule("Player"):CheckPaladinBlessings()
	self:CheckGear()
	self:AnalyzeBuffResults()
	self:RenderTooltip()
	self:UpdateDisplay()

	if self.isBuffEnough and not self.isBuffWarning then
		if self.debug then self:debug("Buff enough") end
	elseif self.isBuffEnough and self.isBuffWarning then
		if self.debug then self:debug("Buff enough with warning") end
	else
		if self.debug then self:debug("Not buff enough") end
	end

end


--[[ ---------------------------------------------------------------------------
	 Iterate through party/raid to see what we have to work with
----------------------------------------------------------------------------- ]]
function BuffEnough:ScanRaidParty()

	if self.debug then self:debug("Checking raid/party makeup") end

	self.raidClassCount = {DRUID = 0, HUNTER = 0, MAGE = 0, PALADIN = 0, PRIEST = 0, ROGUE = 0, SHAMAN = 0, WARLOCK = 0, WARRIOR = 0}
	self.partyClassCount = {DRUID = 0, HUNTER = 0, MAGE = 0, PALADIN = 0, PRIEST = 0, ROGUE = 0, SHAMAN = 0, WARLOCK = 0, WARRIOR = 0}
	self.talentsAvailable = {}

	local unit = nil
	local groupType = nil
	local groupSize = 0
	local startNum = 0
	local playerZone = GetRealZoneText()

	if GetNumRaidMembers() > 0 then
		groupType = "raid"
		groupSize = GetNumRaidMembers()
		startNum = 1
	elseif GetNumPartyMembers() > 0 then
		groupType = "party"
		groupSize = GetNumPartyMembers()
	end

	for i = startNum, groupSize do

		if i == 0 then
			unit = "player"
		else
			unit = groupType..i
		end

		if UnitExists(unit) then

			local unitClass = select(2, UnitClass(unit))
			local unitZone = nil

			if ((groupType == "raid") and (UnitInRaid(unit))) then
				unitZone = select(7, GetRaidRosterInfo(i))
			end

			if self.spam then self:spam("%s %s %s", unit, tostring(unitZone), tostring(unitClass)) end

			if ((not unitZone or unitZone == playerZone) and unitClass) then

				self.raidClassCount[unitClass] = self.raidClassCount[unitClass] + 1

				if (UnitInParty(unit)) then
					self.partyClassCount[unitClass] = self.partyClassCount[unitClass] + 1
				end

				if (unitClass == "PRIEST" or unitClass == "PALADIN") then

					self:GetTalents(unit)

				end

			end

		end

	end

end


--[[ ---------------------------------------------------------------------------
	 Check buffs
----------------------------------------------------------------------------- ]]
function BuffEnough:CheckBuffs()

	if self.debug then self:debug("Checking buffs") end

	-- Initialize
	local hasBattle = false
	local hasGuardian = false
	local checkingConsumables = (GetNumRaidMembers() > 0) or (not self:GetProfileParam("consumablesinraid"))
	local playerPowerType = UnitPowerType("player")

	-- What buffs do we actually have
	local i = 1
	while true do
	
		local buffIndex = select(1, GetPlayerBuff(i, "HELPFUL"))
		if buffIndex == 0 then break end
		
		local buff = select(1, GetPlayerBuffName(buffIndex))
		local timeLeft = GetPlayerBuffTimeLeft(buffIndex)

		-- Determine duration for buff, if we've seen this before
		local duration = timeLeft + 1
		
		if self.durations[buff] and duration < self.durations[buff] then
			duration = self.durations[buff]
		else
			self.durations[buff] = duration
		end

		-- Map if this a longer version of a buff (eg. greater blessing vs regular blessing)
		if self.spellMap[buff] then

			if self.spam then self:spam("Remapped '%s'", buff) end
			buff = self.spellMap[buff]

		end

		-- Special check for flask vs battle/guardian elixirs
		if not self.knownSpells[buff] and checkingConsumables and self:GetProfileParam("flask") then

			G:SetPlayerBuff(buffIndex)

			if (string.find(buff, L["Flask of"]) or
				string.find(buff, L["of Shattrath"]) or
				buff == self.flasks["Supreme Power"] or
				buff == self.flasks["Distilled Wisdom"] or
				buff == self.flasks["Chromatic Wonder"] or
				buff == self.flasks["Chromatic Resistance"])
		   then
				hasBattle = true
				hasGuardian = true
				if self.trace then self:trace("Found flask") end
			elseif G:Find(L["Battle Elixir"]) then
				hasBattle = true
				if self.trace then self:trace("Found battle elixir") end
			elseif G:Find(L["Guardian Elixir"]) then
				hasGuardian = true
				if self.trace then self:trace("Found guardian elixir") end
			else
				self:TrackItem(L["Buffs"], buff, true, false, false, duration, timeLeft)
			end

		else
			self:TrackItem(L["Buffs"], buff, true, false, false, duration, timeLeft)
		end
		
		i = i + 1

	end

	-- What buffs should we expect (or not expect) in general
	self:TrackItem(L["Buffs"], self.spells["Crusader Aura"], false, false, true)
	self:TrackItem(L["Buffs"], self.spells["Aspect of the Pack"], false, false, true)

	if self.playerIsTank then
		self:TrackItem(L["Buffs"], self.spells["Blessing of Salvation"], false, false, true)
	end

	if self.raidClassCount["DRUID"] > 0 then
		self:TrackItem(L["Buffs"], self.spells["Mark of the Wild"], false, true)
	end

	if (self.raidClassCount["MAGE"] > 0 and playerPowerType == 0) then
		self:TrackItem(L["Buffs"], self.spells["Arcane Intellect"], false, true)
	end

	if self.raidClassCount["PRIEST"] > 0 then
		self:TrackItem(L["Buffs"], self.spells["Shadow Resistance Aura"], false, false, true)
		self:TrackItem(L["Buffs"], self.spells["Power Word: Fortitude"], false, true)

		if (self.talentsAvailable[self.spells["Divine Spirit"]] and playerPowerType == 0) then
			self:TrackItem(L["Buffs"], self.spells["Divine Spirit"], false, true)
		end
	end

	if checkingConsumables then

		if self:GetProfileParam("flask") then
			if hasBattle and not hasGuardian then
				self:TrackItem(L["Consumables"], self.spells["Guardian Elixir"], false, true)
			elseif hasGuardian and not hasBattle then
				self:TrackItem(L["Consumables"], self.spells["Battle Elixir"], false, true)
			elseif not hasBattle and not hasGuardian then
				self:TrackItem(L["Consumables"], self.spells["Flask/Elixirs"], false, true)
			end
		end

		if self:GetProfileParam("food") then
			self:TrackItem(L["Consumables"], self.spells["Well Fed"], false, true)
		end
	end

end


--[[ ---------------------------------------------------------------------------
	 Check gear
----------------------------------------------------------------------------- ]]
function BuffEnough:CheckGear()

	if self.debug then self:debug("Checking gear") end

	local twoHander = false

	local itemLink = GetInventoryItemLink("player", GetInventorySlotInfo("MainHandSlot"))
	if (itemLink and select(9,GetItemInfo(itemLink)) == "INVTYPE_2HWEAPON") then
		twoHander = true
	end

	-- Iterate over gear slots, check for missing/broken/almost broken items
	for _,slot in ipairs(invSlots) do

		local slotId = select(1, GetInventorySlotInfo(slot))

		if not GetInventoryItemQuality("player", slotId) then

			if ((slot ~= "SecondaryHandSlot") or (not twoHander)) then
				self:TrackItem(L["Gear"], L[slot], false, true)
			end

		elseif GetInventoryItemBroken("player", slotId) then
			self:TrackItem(L["Gear"], L[slot], false, true, false, nil, nil, nil, L["Broken"])
		else
			local value, max = GetInventoryItemDurability(slotId)
			local percent = 100

		if max and max ~= 0 and value then
		percent = (value / max) * 100
		end

			if percent < 10 then
				self:TrackItem(L["Gear"], L[slot], false, true, false, nil, nil, nil, L["Low"])
			end
		end

	end

	-- Check for temporary weapon enchants
	local checkingConsumables = ((GetNumRaidMembers() > 0) or (not self:GetProfileParam("consumablesinraid")))
	
	if checkingConsumables and self:GetProfileParam("weapon") then
	
		if OffhandHasWeapon() then
			self:TrackItem(L["Consumables"], L["Offhand Buff"], false, true)
		end
		
		if ((self.partyClassCount["SHAMAN"] > 0) and self:GetModule("Player"):CanBenefitFromWF()) then
			self:TrackItem(L["Consumables"], L["Mainhand Buff"], false, false, true)
		else
			self:TrackItem(L["Consumables"], L["Mainhand Buff"], false, true)
		end
		
	end
		
	local hasMHEnchant, mhExp, _, hasOHEnchant, ohExp = GetWeaponEnchantInfo()
	
	if mhExp then mhExp = mhExp / 1000 end
	if ohExp then ohExp = ohExp / 1000 end
	
	if hasOHEnchant then
		self:TrackItem(L["Consumables"], L["Offhand Buff"], true, false, false, 3600, ohExp)
	end

	if hasMHEnchant then
	
		G:SetInventoryItem("player", select(1,GetInventorySlotInfo("MainHandSlot")))
		local hasWindfury = G:Find(L["Windfury"])
		
		if not hasWindfury then
			self:TrackItem(L["Consumables"], L["Mainhand Buff"], true, false, false, 3600, mhExp)
		end
		
	end
	
	-- Check for temporary chest enchants
	if checkingConsumables and self:GetProfileParam("chest") then
		G:SetInventoryItem("player", select(1,GetInventorySlotInfo("ChestSlot")))
		if not G:Find(L["Rune of Warding"]) then
		    self:TrackItem(L["Consumables"], L["Rune of Warding"], false, true)
		end
	end
	
	-- Check for riding crop items
	if IsEquippedItem(self.spells["Riding Crop"]) and not IsMounted() then
		self:TrackItem(L["Gear"], self.spells["Riding Crop"], true, false, true)
	end

	if IsEquippedItem(self.spells["Skybreaker Whip"]) and not IsMounted() then
		self:TrackItem(L["Gear"], self.spells["Skybreaker Whip"], true, false, true)
	end

	-- If shadow resistance is <= 120, then we're probably not stacking
	-- shadow resist and don't mean to be wearing the medallion
	if (IsEquippedItem(self.spells["Blessed Medallion of Karabor"]) and
		select(2, UnitResistance("player", 5)) <= 120)
	then
		self:TrackItem(L["Gear"], self.spells["Blessed Medallion of Karabor"], true, false, true)
	end

	-- Check for fishing poles
	if IsEquippedItemType("Fishing Poles") then
		self:TrackItem(L["Gear"], self.spells["Fishing Pole"], true, false, true)
	end

end


--[[ ---------------------------------------------------------------------------
	 Analyze buff results
----------------------------------------------------------------------------- ]]
function BuffEnough:AnalyzeBuffResults()

	if self.debug then self:debug("Analyzing buff results") end
	
	for b,v in pairs(self.trackedItems) do

		local status = nil
	
		if (v.isExpected and not v.isPresent) then
			status = L["Missing"]
		elseif (v.isUnexpected and v.isPresent) then
			status = L["Unexpected"]
		elseif (v.isExpected and v.isCloseToExpire) then
			status = L["Low"]
		end
		
		if status then
		
			if v.statusOverride then
				status = v.statusOverride
			end
		
			if not self.results[v.category] then
				self.results[v.category] = {}
			end
		
			self.results[v.category][b] = status
			
		end
		
		if self.trace then self:trace("Analyzing buff %s '%s': %s, %s, %s, %s, %s: %s",
									  v.category, b, tostring(v.isPresent), tostring(v.isExpected),
									  tostring(v.isUnexpected), tostring(v.isCloseToExpire),
									  tostring(v.statusOverride), tostring(status)) end

	end
		
end


--[[ ---------------------------------------------------------------------------
	 As we discover buffs that we should have, or buffs that we do have,
	 they get recorded here
----------------------------------------------------------------------------- ]]
function BuffEnough:TrackItem(category, itemName, isPresent, isExpected, isUnexpected, duration, timeLeft, dontReport, statusOverride)

	if not self.trackedItems[itemName] then
		self.trackedItems[itemName] = {}
		self.trackedItems[itemName].category = category
		self.trackedItems[itemName].statusOverride = statusOverride
		self.trackedItems[itemName].isCloseToExpire = false
		self.trackedItems[itemName].dontReport = false
	end

	self.trackedItems[itemName].isPresent = self.trackedItems[itemName].isPresent or isPresent
	self.trackedItems[itemName].isExpected = self.trackedItems[itemName].isExpected or isExpected
	self.trackedItems[itemName].isUnexpected = self.trackedItems[itemName].isUnexpected or isUnexpected
	self.trackedItems[itemName].dontReport = self.trackedItems[itemName].dontReport or dontReport
	
	if (duration and duration > 0 and
		timeLeft and timeLeft > 0 and
		self:GetProfileParam("warn") and
		(self:GetProfileParam("warnthreshold") * 60) >= timeLeft and
		(self:GetProfileParam("warnmin") == 0 or (self:GetProfileParam("warnmin") * 60) <= duration))
	then
		self.trackedItems[itemName].isCloseToExpire = true
	end
		
	if self.trace then self:trace("Tracking item %s '%s': %s, %s, %s, %s (%s / %s)",
								  category, itemName, tostring(isPresent), tostring(isExpected), tostring(isUnexpected),
								  tostring(self.trackedItems[itemName].isCloseToExpire), tostring(timeLeft), tostring(duration)) end

end


--[[ ---------------------------------------------------------------------------
	 Takes a results table from any kind of check and formats it for the
	 tooltip
----------------------------------------------------------------------------- ]]
function BuffEnough:RenderTooltip()

	for heading, res in pairs(self.results) do

		if next(res) then	 

			self.tooltip = self.tooltip.."\n\n"..heading

			for buff,status in pairs(res) do

				if self.debug then self:debug("%s %s", status, buff) end
				
				self.tooltip = self.tooltip.."\n  "..status.." |cffffffff"..buff
				
				if (UnitExists(self.lastBuffer[buff])) then
					self.tooltip = self.tooltip.." ("..self.lastBuffer[buff]..")|r"
				else
					self.tooltip = self.tooltip.."|r"
				end

				if warnings[status] then
					self.isBuffWarning = true
				else
					self.isBuffEnough = false
				end

			end

		end

	end

end


--[[ ---------------------------------------------------------------------------
	 Display missing buffs in party/raid chat
----------------------------------------------------------------------------- ]]
function BuffEnough:PrintResults()

	local dest = nil
	local reportedBuff = false

	if GetNumRaidMembers() > 0 then
		dest = "RAID"
	elseif GetNumPartyMembers() > 0 then
		dest = "PARTY"
	end

	self:DoPrint(L["BuffEnough status for "]..UnitName("player")..":", dest)
	
	if self.results[L["Buffs"]] then	

		for buff,status in pairs(self.results[L["Buffs"]]) do
		    if not self.trackedItems[buff].dontReport then
				self:DoPrint("  "..status.." "..buff, dest)
				reportedBuff = true
			end
		end

	end

	if not reportedBuff then
		self:DoPrint(L["All buffs accounted for!"], dest)
	end
	
end


--[[ ---------------------------------------------------------------------------
	 Whisper missing buffs to the last person in party/raid to cast it
----------------------------------------------------------------------------- ]]
function BuffEnough:WhisperResults()

	local whisperedBuff = false
	
	if self.results[L["Buffs"]] then	

		for buff,status in pairs(self.results[L["Buffs"]]) do
		    if not self.trackedItems[buff].dontReport then
		    	whisperedBuff = true
		    	if UnitExists(self.lastBuffer[buff]) then
					SendChatMessage(L["BuffEnough"]..": "..status.." "..buff, "WHISPER", nil, self.lastBuffer[buff])
				else
					self:DoPrint(L["No buffer known for "]..buff)
				end
			end
		end

	end

	if not whisperedBuff then
		self:DoPrint(L["BuffEnough status for "]..UnitName("player")..":")
		self:DoPrint(L["All buffs accounted for!"])
	end
	
end


--[[ ---------------------------------------------------------------------------
	 Determine where to do text output
----------------------------------------------------------------------------- ]]
function BuffEnough:DoPrint(string, loc)

    if loc then
    	SendChatMessage(string, loc)
    else
    	DEFAULT_CHAT_FRAME:AddMessage(string);
    end

end

--[[ ---------------------------------------------------------------------------
	 Determine if the player is a tank according to CTRaid or oRA
----------------------------------------------------------------------------- ]]
function BuffEnough:CheckTank()

	local tanks = nil
	self.playerIsTank = false

	if oRA and oRA.maintanktable then
		tanks = oRA.maintanktable
	elseif CT_RA_MainTanks then
		tanks = CT_RA_MainTanks
	end

	if tanks then
		for _, name in pairs(tanks) do
			if name == UnitName("player") then
				self.playerIsTank = true
			end
		end
	end

	if self.trace then self:trace("Player %s a tank", (self.playerIsTank and "is" or "is not")) end

end


--[[ ---------------------------------------------------------------------------
	 Queue up a talent query for the given unit (or do it right away if its
	 for the player)
----------------------------------------------------------------------------- ]]
function BuffEnough:GetTalents(unit)

	local name = UnitName(unit)

	if not self.talents[name] then

		if self.trace then self:trace("Get new talents for %s", name) end

	if UnitIsUnit(unit, "player") then
		self:TalentQuery_Ready(_, name)
	else
		TQ:Query(unit)
	end

	elseif next(self.talents[name]) then

		if self.trace then self:trace("Existing talents for %s", name) end

		for t,_ in pairs(self.talents[name]) do
			if self.trace then self:trace("%s is available", t) end
			self.talentsAvailable[t] = true
		end

	end

end


--[[ ---------------------------------------------------------------------------
	 Callback function for unit talent inspection, called from LibTalentQuery
	 after we call Query() and the ready event fires
----------------------------------------------------------------------------- ]]
function BuffEnough:TalentQuery_Ready(_, name)

	self.talents[name] = {}
	local unitClass = select(2, UnitClass(name))
	local isNotPlayer = not UnitIsUnit(name, "player")

	if self.trace then self:trace("Talent query for %s %s %s", unitClass, name, tostring(isNotPlayer)) end

	if unitClass == "PALADIN" then
		self:DoTalentQuery(name, self.spells["Blessing of Kings"], 2, 6, isNotPlayer)
		self:DoTalentQuery(name, self.spells["Blessing of Sanctuary"], 2, 14, isNotPlayer)
	elseif unitClass == "PRIEST" then
		self:DoTalentQuery(name, self.spells["Divine Spirit"], 1, 14, isNotPlayer)
	end

	self:ScanRaidParty()

end


--[[ ---------------------------------------------------------------------------
	 Perform and record the specified talent query
----------------------------------------------------------------------------- ]]
function BuffEnough:DoTalentQuery(name, talent, tabIndex, talentIndex, isNotPlayer)

	local points = select(5, GetTalentInfo(tabIndex, talentIndex, isNotPlayer))
	if self.trace then self:trace("%s %s: %d", name, talent, points) end

	if points > 0 then
		self.talents[name][talent] = points
	end

end


--[[ ---------------------------------------------------------------------------
	 Record who last cast a buff on the player
----------------------------------------------------------------------------- ]]
function BuffEnough:RecordLastBuffer(_, _, eventType, srcGUID, srcName, _, dstGUID, dstName, _, _, spellName)

	if (eventType == "SPELL_CAST_SUCCESS" and
	    dstGUID and 
	    self.knownSpells[spellName] and
	    srcGUID ~= UnitGUID("player") and
	    (dstGUID == UnitGUID("player") or
	     (self.spellMap[spellName] and UnitInParty(dstName)) or
	     (self.blessings[spellName] and select(2, UnitClass(dstName)) == select(2, UnitClass("player")))))
	then
	
		if self.spellMap[spellName] then
			spellName = self.spellMap[spellName]
		end
	
		self.lastBuffer[spellName] = srcName
	    if self.debug then self:debug("Last buffer for %s: %s", tostring(spellName), tostring(srcName)) end
	end
	
end