--[[ ---------------------------------------------------------------------------

BuffEnough: personal buff monitor.

BuffEnough is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

BuffEnough is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BuffEnough.  If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]


local L = LibStub("AceLocale-3.0"):GetLocale("BuffEnough")


local prototype = {}
prototype.playerClass = select(2, UnitClass("player"))


--[[ ---------------------------------------------------------------------------
     Check paladin blessings for class
----------------------------------------------------------------------------- ]]
function prototype:CheckPaladinBlessings()

    local paladinBlessings = {}

    if BuffEnough:GetProfileParam("overrideblessings") then
        for i=1,6 do
            paladinBlessings[i] = BuffEnough:GetProfileParam("blessing"..i)
        end
    else
        paladinBlessings = self.GetPaladinBlessingList()
    end

    local i = 1

    for _,blessing in ipairs(paladinBlessings) do

        if (i > BuffEnough.raidClassCount["PALADIN"]) then break end

        if blessing ~= L["None"] then
            if (((blessing == BuffEnough.spells["Blessing of Salvation"]) and (not BuffEnough.playerIsTank)) or
                ((blessing == BuffEnough.spells["Blessing of Sanctuary"]) and (BuffEnough.talentsAvailable[BuffEnough.spells["Blessing of Sanctuary"]])) or
                ((blessing == BuffEnough.spells["Blessing of Kings"]) and (BuffEnough.talentsAvailable[BuffEnough.spells["Blessing of Kings"]])) or
                (blessing ~= BuffEnough.spells["Blessing of Salvation"] and
                 blessing ~= BuffEnough.spells["Blessing of Sanctuary"] and
                 blessing ~= BuffEnough.spells["Blessing of Kings"]))
            then

                BuffEnough:TrackItem(L["Buffs"], blessing, false, true)
                i = i + 1

            end
        end

    end

end

--[[ ---------------------------------------------------------------------------
     Check pet buffs
----------------------------------------------------------------------------- ]]
function prototype:CheckPetBuffs()

    if UnitExists("pet") and
       BuffEnough:GetProfileParam("petfood") and
       ((GetNumRaidMembers() > 0) or (not BuffEnough:GetProfileParam("consumablesinraid")))
    then
    
		BuffEnough:TrackItem(L["Pet"], BuffEnough.spells["Well Fed"], false, true)
    
		local i = 1
		local petbuff = UnitBuff("pet", i);
		while petbuff do
		    BuffEnough:TrackItem(L["Pet"], petbuff, true)
			i = i + 1;
  			petbuff = UnitBuff("pet", i);
		end
		
    end

end

--[[ ---------------------------------------------------------------------------
     Functions to be overriden by the prototype implementation
----------------------------------------------------------------------------- ]]
prototype.CheckClassBuffs = function() end
prototype.CanBenefitFromWF = function() return false end
prototype.GetPaladinBlessingList = function() end


--[[ ---------------------------------------------------------------------------
     Loads the module for the prototype
----------------------------------------------------------------------------- ]]
function BuffEnough:GetOrCreateModule(t)	
    return self:GetModule(t, true) or self:NewModule(t, self.ClassPrototype)
end


BuffEnough.ClassPrototype = prototype
