--[[ ---------------------------------------------------------------------------

BuffEnough: personal buff monitor.

BuffEnough is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

BuffEnough is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BuffEnough.	If not, see <http://www.gnu.org/licenses/>.

----------------------------------------------------------------------------- ]]


local L = LibStub("AceLocale-3.0"):GetLocale("BuffEnough")
local C = LibStub("AceConfigDialog-3.0")


--[[ ---------------------------------------------------------------------------
	 Create dropdowns
----------------------------------------------------------------------------- ]]
local logLevelsDropdown = {}

if BuffEnough.logLevels then
	for logname,id in pairs(BuffEnough.logLevels) do
		logLevelsDropdown[id] = L[logname]
	end
end

local blessingsDropdown = {}
blessingsDropdown[BuffEnough.spells["Blessing of Kings"]] = BuffEnough.spells["Blessing of Kings"]
blessingsDropdown[BuffEnough.spells["Blessing of Light"]] = BuffEnough.spells["Blessing of Light"]
blessingsDropdown[BuffEnough.spells["Blessing of Might"]] = BuffEnough.spells["Blessing of Might"]
blessingsDropdown[BuffEnough.spells["Blessing of Salvation"]] = BuffEnough.spells["Blessing of Salvation"]
blessingsDropdown[BuffEnough.spells["Blessing of Sanctuary"]] = BuffEnough.spells["Blessing of Sanctuary"]
blessingsDropdown[BuffEnough.spells["Blessing of Wisdom"]] = BuffEnough.spells["Blessing of Wisdom"]
blessingsDropdown[L["None"]] = L["None"]


--[[ ---------------------------------------------------------------------------
	 Config options
----------------------------------------------------------------------------- ]]
BuffEnough.options = {
	name = L["BuffEnough"],
	handler = BuffEnough,
	type = "group",
	args = {
		config = {
			type = "execute",
			name = L["Config"],
			desc = L["Toggle the configuration dialog"],
			func = "ToggleConfigDialog",
		   	guiHidden = true
		},
	   	show = {
	   		type = "execute",
	   		name = L["Show"],
	   		desc = L["Toggle the display"],
	   		func = "ToggleDisplay",
		   	guiHidden = true
		},
		general = {
			type = "group",
			name = L["General"],
			desc = L["General options"],
			order = 0,
			set = "SetProfileParam",
			get = "GetProfileParam",
			args = {
				warn = {
					type = "toggle",
					name = L["Warn"],
					desc = L["Display shows warning color when you are buff enough but one or more buffs is about to expire"],
					width = "full",
					order = 1,
					set = function(info, v)
							BuffEnough:SetProfileParam("warn", v)
							BuffEnough:UpdateDisplay()
				  		end	
				},
				warnmin = {
					type = "range",
					name = L["Warn Minimum Buff Time"],
					desc = L["The minimum buff duration in minutes before we check for expiration. Set to 0 for no minimum"],
					min = 0,
					max = 120,
					step = 1,
					bigStep = 1,
					width = "full",
					order = 2,
					disabled = function() return not BuffEnough:GetProfileParam("warn") end,
				},
				warnthreshold = {
					type = "range",
					name = L["Warn Threshold"],
					desc = L["The number of minutes left on the buff before we warn"],
					min = 0,
					max = 30,
					step = 1,
					bigStep = 1,
					width = "full",
					order = 3,
					disabled = function() return not BuffEnough:GetProfileParam("warn") end ,
				},
				fade = {
					type = "range",
					name = L["Fade"],
					desc = L["Fade frame after this many seconds of being buff enough. Set to 0 for no fading"],
					min = 0,
					max = 60,
					step = 1,
					bigStep = 1,
					width = "full",
					order = 4,
				},
				logLevel = {
					type = "select",
					name = L["Log Level"],
					desc = L["Determines the amount of output from the addon"],
					order = -1,
					values = logLevelsDropdown,
					disabled = function() return not LibStub("LibLogger-1.0", true) end,
				},
			},
		},
		blessings = {
			type = "group",
			name = L["Blessings"],
			desc = L["Blessings options"],
			set = "SetProfileParam",
			get = "GetProfileParam",
			args = {
				override = {
					type = "toggle",
					name = L["Override"],
					desc = L["Overrides the default paladin blessing priority for your class/spec and uses the priority you specify."],
					width = "full",
					order = 0,
					get = function(info) return BuffEnough:GetProfileParam("overrideblessings") end,
					set = function(info, v) BuffEnough:SetProfileParam("overrideblessings", v) end	
				},
				blessing1 = {
					type = "select",
					name = L["Blessing"].."1",
					disabled = function() return not BuffEnough:GetProfileParam("overrideblessings") end,
					values = blessingsDropdown,
				},
				blessing2 = {
					type = "select",
					name = L["Blessing"].."2",
					disabled = function() return not BuffEnough:GetProfileParam("overrideblessings") end,
					values = blessingsDropdown,
				},
				blessing3 = {
					type = "select",
					name = L["Blessing"].."3",
					disabled = function() return not BuffEnough:GetProfileParam("overrideblessings") end,
					values = blessingsDropdown,
				},
				blessing4 = {
					type = "select",
					name = L["Blessing"].."4",
					disabled = function() return not BuffEnough:GetProfileParam("overrideblessings") end,
					values = blessingsDropdown,
				},
				blessing5 = {
					type = "select",
					name = L["Blessing"].."5",
					disabled = function() return not BuffEnough:GetProfileParam("overrideblessings") end,
					values = blessingsDropdown,
				},
				blessing6 = {
					type = "select",
					name = L["Blessing"].."6",
					disabled = function() return not BuffEnough:GetProfileParam("overrideblessings") end,
					values = blessingsDropdown,
				},
			},
		},
		consumables = {
			type = "group",
			name = L["Consumables"],
			desc = L["Consumable options"],
			set = "SetProfileParam",
			get = "GetProfileParam",
			args = {
				flask = {
					type = "toggle",
					name = L["Check Flask/Elixirs"],
					desc = L["Check whether or not you have either a flask or Guardian and Battle elixirs"],
					width = "full",
					order = 2,
				},
				food = {
					type = "toggle",
					name = L["Check Food"],
					desc = L["Check whether or not you have a food buff"],
					width = "full",
					order = 4,
				},
				petfood = {
					type = "toggle",
					name = L["Check Pet Food"],
					desc = L["Check whether or not your pet has a food buff"],
					width = "full",
					order = 5,
				},
				weapon = {
					type = "toggle",
					name = L["Check Weapon"],
					desc = L["Check whether or not you have a temporary weapon enchant"],
					width = "full",
					order = 3,
				},
				chest = {
					type = "toggle",
					name = L["Check Chest"],
					desc = L["Check whether or not your chest has a Rune of Warding"],
					width = "full",
					order = 6,
				},
				inraid = {
					type = "toggle",
					name = L["Only Check In Raid"],
					desc = L["Only check consumable use when in a raid"],
					width = "full",
					order = 1,
					get = function(info) return BuffEnough:GetProfileParam("consumablesinraid") end,
					set = function(info, v) BuffEnough:SetProfileParam("consumablesinraid", v) end
				},
			},
		},
		display = {
			type = "group",
			name = L["Display"],
			desc = L["Display options"],
			args = {
				lock = {
					type = "toggle",
					name = L["Lock"],
					desc = L["Lock the display"],
					order = 0,
					width = "full",
					get = function(info) return BuffEnough:GetProfileParam("lock") end,
					set = function(info, v)
					  		BuffEnough:SetProfileParam("lock", v)
					  		BuffEnough:UpdateDisplay()
				  		end	
				},
				scale = {
					type = "range",
					name = L["Scale"],
					desc = L["Scale of the display"],
					min = 1,
					max = 150,
					step = 1,
					bigStep = 1,
					order = 1,
					width = "full",
					get = function(info) return BuffEnough:GetProfileParam("scale") end,
					set = function(info, v)
							BuffEnough:SetProfileParam("scale", v)
			  				BuffEnough:UpdateDisplay()
						end						
				},
				alpha = {
					type = "range",
					name = L["Opacity"],
					desc = L["Opacity of the display"],
					min = 0,
					max = 1,
					step = 0.01,
					bigStep = 0.05,
					order = 2,
					width = "full",
					get = function(info) return BuffEnough:GetProfileParam("alpha") end,
					set = function(info, v)
				 			 BuffEnough:SetProfileParam("alpha", v)
				  			BuffEnough:UpdateDisplay()
						end	
				},
				buffcolor = {
					type = "color",
					name = L["Buff Color"],
					desc = L["Color to display when you are buff enough"],
					hasAlpha = false,
					width = "full",
					get = function(info)
							local r = BuffEnough:GetProfileParam("buffcolorr")
							local g = BuffEnough:GetProfileParam("buffcolorg")
							local b = BuffEnough:GetProfileParam("buffcolorb")
							return r, g, b
					end,
					set = function(info, r, g, b)
							BuffEnough:SetProfileParam("buffcolorr", r)
							BuffEnough:SetProfileParam("buffcolorg", g)
							BuffEnough:SetProfileParam("buffcolorb", b) 
							BuffEnough:UpdateDisplay()
						end
				},
				unbuffcolor = {
					type = "color",
					name = L["Unbuff Color"],
					desc = L["Color to display when you are not buff enough"],
					hasAlpha = false,
					width = "full",
					get = function(info)
							local r = BuffEnough:GetProfileParam("unbuffcolorr")
							local g = BuffEnough:GetProfileParam("unbuffcolorg")
							local b = BuffEnough:GetProfileParam("unbuffcolorb")
							return r, g, b
						end,
					set = function(info, r, g, b)
							BuffEnough:SetProfileParam("unbuffcolorr", r)
							BuffEnough:SetProfileParam("unbuffcolorg", g)
							BuffEnough:SetProfileParam("unbuffcolorb", b) 
							BuffEnough:UpdateDisplay()
						end
				},
				warnbuffcolor = {
					type = "color",
					name = L["Warn Buff Color"],
					desc = L["Color to display when you are buffs are at a warning level"],
					hasAlpha = false,
					width = "full",
					get = function(info)
							local r = BuffEnough:GetProfileParam("warnbuffcolorr")
							local g = BuffEnough:GetProfileParam("warnbuffcolorg")
							local b = BuffEnough:GetProfileParam("warnbuffcolorb")
							return r, g, b
						end,
					set = function(info, r, g, b)
							BuffEnough:SetProfileParam("warnbuffcolorr", r)
							BuffEnough:SetProfileParam("warnbuffcolorg", g)
							BuffEnough:SetProfileParam("warnbuffcolorb", b) 
							BuffEnough:UpdateDisplay()
						end
				},

			},
		},
	},
}


--[[ ---------------------------------------------------------------------------
	 Config defaults
----------------------------------------------------------------------------- ]]
BuffEnough.defaults = {
	profile = {
		show = true,
		logLevel = 0,
		warn = true,
		warnmin = 30,
		warnthreshold = 5,
		fade = 0,
		overrideblessings = false,
		flask = true,
		food = true,
		petfood = false,
		weapon = true,
		chest = false,
		consumablesinraid = true,
		lock = false,
		alpha = 1,
		scale = 100,
		buffcolorr = 0,
		buffcolorg = .75,
		buffcolorb = 0,
		unbuffcolorr = .75,
		unbuffcolorg = 0,
		unbuffcolorb = 0,
		warnbuffcolorr = .75,
		warnbuffcolorg = .75,
		warnbuffcolorb = 0,
	},
}

local blessingList = BuffEnough:GetModule("Player"):GetPaladinBlessingList()
for i,blessing in pairs(blessingList) do
	BuffEnough.defaults.profile["blessing"..i] = blessingList[i]
end
if #blessingList < 6 then BuffEnough.defaults.profile["blessing"..6] = L["None"] end

--[[ ---------------------------------------------------------------------------
	 Toggle the config window
----------------------------------------------------------------------------- ]]
function BuffEnough:ToggleConfigDialog()

   local frame = C.OpenFrames[L["BuffEnough"]]

   if frame then
	  C:Close(L["BuffEnough"])
   else
	  C:Open(L["BuffEnough"])
	  self:SetStatusText(string.format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set the text displayed at the bottom of the config window
----------------------------------------------------------------------------- ]]
function BuffEnough:SetStatusText(text)

   local frame = C.OpenFrames[L["BuffEnough"]]

   if frame then
	  frame:SetStatusText(text)
   end

end


--[[ ---------------------------------------------------------------------------
	 Update config display on a profile change
----------------------------------------------------------------------------- ]]
function BuffEnough:OnProfileChanged(event, newdb)

   if event ~= "OnProfileDeleted" then
	  if LibStub("LibLogger-1.0", true) then self:SetLogLevel(self.db.profile.logLevel) end
	  self:SetStatusText(string.format(L["Profile: %s"], self.db:GetCurrentProfile()))
   end

end


--[[ ---------------------------------------------------------------------------
	 Set a saved variable config option
----------------------------------------------------------------------------- ]]
function BuffEnough:SetProfileParam(var, value)

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.trace then self:trace("Setting parameter %s to %s.", varName, tostring(value)) end

   self.db.profile[varName] = value

   if varName == "logLevel" and LibStub("LibLogger-1.0", true) then
	  self:SetLogLevel(value)
   end

end


--[[ ---------------------------------------------------------------------------
	 Get a saved variable config option
----------------------------------------------------------------------------- ]]
function BuffEnough:GetProfileParam(var) 

   local varName = nil

   if type(var) == "string" then
	   varName = var
   else
	   varName = var[#var]
   end

   if self.spam then self:spam("Getting parameter %s as %s", varName, tostring(self.db.profile[varName])) end

   return self.db.profile[varName]

end